#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>

#include "vector.h"
#include "spectra.h"
#include "matrix.h"


#ifdef __cplusplus
extern "C" {
#endif

#include "./lib/NCCAPixmap.h"
#include "./lib/PixFileIO.h"

#ifdef __cplusplus
}
#endif


#define WIDTH 720
#define HIEGHT 576

#define MAXF 7.8
#define MINF 3.8
#define STEPF 0.1

using namespace std;

char *holes_fname="./holes.obj";
char *targets_fname="./targets/targets.obj";
char *xbar ="./xyf_functions/xbar.match";
char *ybar ="./xyf_functions/ybar.match";
char *zbar ="./xyf_functions/zbar.match";


double my_distance(amirVector, amirVector);
void LoadData(char *filename, int num, amirVector* &data);
int numb_points(char *filename);
void amplitude_function(amirVector light, amirVector* target ,amirVector* &holes ,int num_h, int num_t, char* filename);
spectra spectra_filter(void);

int main(int argc, char*argv[])
{
	if(argc != 2)
	{
		cerr<<"please specify the name of the output image"<<endl;
			exit(1);
	}


	int numb_holes;
	int numb_target;
	
	
	amirVector *hole_points = NULL;
	amirVector *target_points = NULL;
	amirVector light_point(-84, 0, 0);


	NCCAPixmap image;
	NCCAPixel p;
	
	
	numb_holes = numb_points(holes_fname);
	numb_target = numb_points(targets_fname);
	LoadData(holes_fname, numb_holes, hole_points);
	LoadData(targets_fname, numb_target, target_points);

	amplitude_function(light_point, target_points, hole_points, numb_holes, numb_target,argv[1]);

	if(hole_points != NULL)
		delete [] hole_points;
	if(target_points != NULL)
		delete [] target_points;
		
	return 0;
}


spectra spectra_filter(void)
{

spectra xyz(xbar,ybar,zbar);
return xyz;
}



void amplitude_function(amirVector light, amirVector* target, amirVector* &holes, int num_h, int num_t, char* filename)
{	
	double *x_phase = NULL;
	double *y_phase = NULL;
	double *t_dist = NULL;
	amirVector *xyz_Colour = NULL;
	
	int count = 0;
	for(double i = 3.8; i <= 7.8;i +=  0.1)
	{
		++count;
	}

		//  allocate memory for xyz colour
	xyz_Colour = new amirVector[num_t];
	if (xyz_Colour==NULL) {
	cerr<<"Error: Out of memory"<<endl;
			exit(1);
	}
	

	
	//  allocate memory for distances
	t_dist = new double[num_h];
	if (t_dist==NULL) {
	cerr<<"Out of memory"<<endl;
			exit(1);
	}
	//  allocate memory for x phase
	x_phase = new double[num_h];
	if (x_phase==NULL) {
	cerr<<"Error: Out of memory"<<endl;
			exit(1);
	}
	
	//  allocate memory for y phase
	y_phase = new double[num_h];
	if (y_phase==NULL) {
	cerr<<"Error: Out of memory"<<endl;
			exit(1);
	}

	
	spectra xyz = spectra_filter();
	//xyz.print();
	int countw = 0;
	
	for(int k = 0; k < num_t; ++k)
	{
		countw = 0;
		for (double w = 3.8;w <= 7.8;w += 0.1)
		{
			amirVector amp(0,0,0);
			for(int i = 0; i <num_h ; ++i)
			{
				double shift;
				double scale;
				t_dist[i] = (my_distance(light, holes[i]) + my_distance(target[k], holes[i]));
				shift=t_dist[i]/w;
				shift-=floor(shift);
				x_phase[i] = sin(M_PI*2*(shift));
				y_phase[i] = cos(M_PI*2*(shift));
				amp.x += x_phase[i];
				amp.y += y_phase[i];
				amp.z = 0;

			}
			
 			xyz_Colour[k].x += xyz.get(0,1,countw)*amp.length()/5;
			xyz_Colour[k].y += xyz.get(1,1,countw)*amp.length()/5;
			xyz_Colour[k].z += xyz.get(2,1,countw)*amp.length()/5;
			
		++countw;
		}
		
		xyz_Colour[k].x /= count;
		xyz_Colour[k].y /= count;
		xyz_Colour[k].z /= count;
		xyz_Colour[k].xyzTOrgb();
		xyz_Colour[k].print();
		
	}
	
	
	
	
	NCCAPixmap img;
	img = newPixmap(720, num_t, 3, 8);
	
		for(int i=0;i<img.width;i++)
			for(int j=0;j<img.height;j++)
			{
			NCCAPixel p;
			p.r =xyz_Colour[j].x;
			p.g =xyz_Colour[j].y;
			p.b =xyz_Colour[j].z;
			setPixelColour(img,i,j,p);
			}
	
			
	savePixmap(img, filename);
	string kuickshow ="kuickshow ";
	
	kuickshow.insert(kuickshow.size(),filename);
	cout<< "saving to "<<filename<<endl;
	system(kuickshow.c_str());
	
	if(t_dist != NULL)
		delete [] t_dist;
		
	if(x_phase != NULL)
		delete [] x_phase;
		
	if(y_phase != NULL)
	
	if(xyz_Colour != NULL)
		delete [] xyz_Colour;
		
	
}


int numb_points(char *filename)
{
	int numb_points=0;
	fstream FileIn;
	string LineBuffer;	
	
	FileIn.open(filename,ios::in);
	if(!FileIn.is_open())
		{
		cout<< filename<<" not found"<< endl;
		exit(1);
		}
		
	//count the number of points
	while(getline(FileIn, LineBuffer, '\n'))
	{
		switch(LineBuffer[0])
		{
			case 'v':
				if(LineBuffer[1] == ' ')
				numb_points++;
		}
	}
	FileIn.close();
	return numb_points;
}

void LoadData(char *filename, int num, amirVector* &data)
{
	fstream FileIn;
	string LineBuffer;	
	int i=0;

	//  allocate memory for points
	data = new amirVector[num];
	if (data==NULL) 
	{
	cerr<<"Error: Out of memory"<<endl;
			exit(1);
	}

		
	FileIn.open(filename,ios::in);
	while(getline(FileIn, LineBuffer, '\n'))
	{
		if(LineBuffer[0]=='v'&& LineBuffer[1]==' ')
		{
			char v;
			stringstream ss(LineBuffer);
			ss >> v >> data[i].x >> data[i].y >> data[i].z;
			i++;
		}
	}
	FileIn.close();
}


double my_distance(amirVector v1, amirVector v2)
{
	amirVector v3 = v1-v2;
	double dist = v3.length();
	return dist;
}
