
#ifndef __MATRIX_H__
#define __MATRIX_H__


// symbolic constants 
#define PI  3.141592653590f
using namespace std;

/*
 * convert degrees to radians
 */
inline float degr2rad(float angle) { return angle * PI / 180.0f; }
inline float rad2degr(float angle) { return angle * 180.0f / PI; }

//
// homogenous transformation matrix
//
class matrix {
	// 4x4 matrix
	float data[16];
public:

	// matrix elements
	enum E_ID {  
		_11=0, _12=4, _13= 8, _14=12,
		_21=1, _22=5, _23= 9, _24=13,
		_31=2, _32=6, _33=10, _34=14,
		_41=3, _42=7, _43=11, _44=15
	};

	// constructor
	matrix(
		float v_11, float v_12, float v_13, float v_14,
		float v_21, float v_22, float v_23, float v_24,
		float v_31, float v_32, float v_33, float v_34,
		float v_41, float v_42, float v_43, float v_44
	);

	// constructor
	matrix(const float v[16]);

	// copy constructor
	matrix(const matrix &);

	// default constructor
	matrix();

	// get martix elements
	float getElement(E_ID i) const {
		return data[i];
	}

	// get matrix data
	const float *getData() const {
		return data;
	}

	// set matrix elements
	void setElement(E_ID i, float v) {
		data[i] = v;
	}

	// loads the identity matrix
	void identity();

	// apply transformations
	void translate(float t_x, float t_y, float t_z);
	void scale(float s_x, float s_y, float s_z);
	void rotateX(float angle);  // rotation angle in degrees
	void rotateY(float angle);  // rotation angle in degrees
	void rotateZ(float angle);  // rotation angle in degrees

	// multiply "*this" with "r"
	void mult(const matrix &r);

	// "*this" *= "r"
	matrix& operator*=(const matrix &r) {
		mult(r);
		return *this;
	}
	
	// multiply "*this" with "r"
	matrix operator*(const matrix &r) {
		matrix l = *this;
		l *= r;
		return l;
	}

	matrix& operator=(const matrix& m);

	// print to output stream
	void print(); 
};


#endif // __MATRIX_H__

